@@ -2,21 +2,21 @@
  * Gregorio is a program that translates gabc files to GregorioTeX
  * This file contains the logic for positioning signs on neumes.
  *
- * Copyright (C) 2008-2021 The Gregorio Project (see CONTRIBUTORS.md)
+ * Copyright (C) 2008-2025 The Gregorio Project (see CONTRIBUTORS.md)
  *
  * This file is part of Gregorio.
- * 
+ *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
  * Software Foundation, either version 3 of the License, or (at your option)
  * any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but WITHOUT
  * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  * more details.
- * 
- * You should have received a copy of the GNU General Public License along with 
+ *
+ * You should have received a copy of the GNU General Public License along with
  * this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "config.h"
@@ -90,10 +90,13 @@ OFFSET_CASE(LeadingQuilisma);
 OFFSET_CASE(LeadingOriscus);
 OFFSET_CASE(Flat);
 OFFSET_CASE(FlatParen);
+OFFSET_CASE(FlatSoft);
 OFFSET_CASE(Sharp);
 OFFSET_CASE(SharpParen);
+OFFSET_CASE(SharpSoft);
 OFFSET_CASE(Natural);
 OFFSET_CASE(NaturalParen);
+OFFSET_CASE(NaturalSoft);
 
 static __inline const char *note_before_last_note_case_ignoring_deminutus(
         const gregorio_note *const current_note)
@@ -140,7 +143,7 @@ static __inline const char *last_note_case(
     if (current_glyph->u.notes.liquescentia & L_DEMINUTUS) {
         /*
          * may seem strange, but it is unlogical to typeset a small horizontal
-         * episema at the end of a flexus deminutus 
+         * episema at the end of a flexus deminutus
          */
         return FinalDeminutus;
     }
@@ -382,7 +385,7 @@ static gregorio_vposition advise_positioning(const gregorio_glyph *const glyph,
     case T_PES:
     case T_QUILISMA_PES:
         /*
-         * in the case of a pes, we put the episema just under the bottom note 
+         * in the case of a pes, we put the episema just under the bottom note
          */
         if (i == 1) {
             if (glyph->u.notes.liquescentia & L_INITIO_DEBILIS) {
@@ -436,7 +439,7 @@ static gregorio_vposition advise_positioning(const gregorio_glyph *const glyph,
             if (type == T_FLEXUS_ORISCUS) {
                 note->gtex_offset_case = InitialConnectedOriscus;
             } else {
-                if (note->next && note->u.note.pitch - 
+                if (note->next && note->u.note.pitch -
                         note->next->u.note.pitch == 1) {
                     note->gtex_offset_case = InitialVirga;
                 } else {
@@ -951,18 +954,27 @@ static gregorio_vposition advise_positioning(const gregorio_glyph *const glyph,
         case S_FLAT_PAREN:
             note->gtex_offset_case = FlatParen;
             break;
+        case S_FLAT_SOFT:
+            note->gtex_offset_case = FlatSoft;
+            break;
         case S_SHARP:
             note->gtex_offset_case = Sharp;
             break;
         case S_SHARP_PAREN:
             note->gtex_offset_case = SharpParen;
             break;
+        case S_SHARP_SOFT:
+            note->gtex_offset_case = SharpSoft;
+            break;
         case S_NATURAL:
             note->gtex_offset_case = Natural;
             break;
         case S_NATURAL_PAREN:
             note->gtex_offset_case = NaturalParen;
             break;
+        case S_NATURAL_SOFT:
+            note->gtex_offset_case = NaturalSoft;
+            break;
         default:
             note->gtex_offset_case = last_note_case(glyph,
                     fused_single_note_case(glyph, FinalPunctum, LeadingPunctum),
@@ -1549,6 +1561,7 @@ static __inline int compute_fused_shift(const gregorio_glyph *glyph)
 
     case G_PUNCTUM:
     case G_FLEXA:
+    case G_VIRGA:
         /* these may be fused to the previous note */
         break;
 
@@ -1588,6 +1601,11 @@ static __inline int compute_fused_shift(const gregorio_glyph *glyph)
     gregorio_assert(prev_note->type == GRE_NOTE, compute_fused_shift,
             "previous note wasn't a note", return 0);
 
+    if (prev_note->u.note.shape == S_VIRGA) {
+        /* cannot fuse after a virga */
+        return 0;
+    }
+
     shift = first_note->u.note.pitch - prev_note->u.note.pitch;
     gregorio_assert(shift >= -MAX_AMBITUS && shift <= MAX_AMBITUS,
             compute_fused_shift, "ambitus too large to fuse", return 0);
